package listaCircular.modelo;

public class ListaCircular<T> {
	// Primer elemento de la lista
	private Nodo<T> cabeza;
	// Total de elementos de la lista
	private int tamanio;

	// Constructor por defecto
	public ListaCircular() {
		cabeza = null;
		tamanio = 0;
	}

	/**
	 * Devuelve el tamanio de la lista
	 * 
	 * @return tamanio de la lista
	 */
	public int getTamanio() {
		return tamanio;
	}

	/**
	 * Consulta si la lista esta vacia
	 * 
	 * @return true cuando esta vacia, false cuando no
	 */
	public boolean esVacia() {
		return (cabeza == null);
	}

	/**
	 * Agrega un nuevo nodo al final de la lista
	 * 
	 * @param valor
	 *            valor a agregar
	 */
	public void agregar(T valor) {
		Nodo<T> nuevo = new Nodo<T>();
		nuevo.setValor(valor);
		if (esVacia()) {
			// Como hay un solo nodo, el siguiente apunta al mismo nodo
			nuevo.setSiguiente(nuevo);
			cabeza = nuevo;
		} else {
			// agregar al final de la lista
			Nodo<T> aux = cabeza;
			while (aux.getSiguiente() != cabeza) {
				aux = aux.getSiguiente();
			}
			aux.setSiguiente(nuevo);
			// Enlaza el ultimo nodo con el primero
			nuevo.setSiguiente(cabeza);
		}
		tamanio++;
	}

	/**
	 * inserta un nuevo nodo en la lista
	 * 
	 * @param valor
	 *            valor a agregar
	 * @param pos
	 *            posicion donde se insertara el nodo
	 * @throws PosicionIlegalException
	 *             excepcion en caso que la posicion no exista
	 */
	public void insertar(T valor, int pos) throws PosicionIlegalException {
		// Para insertar la posicion debe estar entre 0 y el tamanio,
		// y ademas, debe haber al menos un dato en la lista
		if (pos >= 0 && pos <= tamanio && tamanio > 0) {
			Nodo<T> nuevo = new Nodo<T>();
			nuevo.setValor(valor);
			// el nodo a insertar esta al comienzo de la lista
			if (pos == 0) {
				// Buscar el ultimo nodo
				Nodo<T> ultimo = cabeza;
				while (ultimo.getSiguiente() != cabeza) {
					ultimo = ultimo.getSiguiente();
				}
				nuevo.setSiguiente(cabeza);
				cabeza = nuevo;
				// Enlazar el ultimo con el primero
				ultimo.setSiguiente(cabeza);
			} else {
				// El nodo a insertar va al final de la lista
				if (pos == tamanio) {
					Nodo<T> aux = cabeza;
					while (aux.getSiguiente() != cabeza) {
						aux = aux.getSiguiente();
					}
					aux.setSiguiente(nuevo);
					// enlazar el ultimo con el primero
					nuevo.setSiguiente(cabeza);
				}
				// el nodo a insertar esta en medio
				else {
					Nodo<T> aux = cabeza;
					for (int i = 0; i <= pos - 2; i++) {
						aux = aux.getSiguiente();
					}
					Nodo<T> siguiente = aux.getSiguiente();
					aux.setSiguiente(nuevo);
					nuevo.setSiguiente(siguiente);
				}
			}
			tamanio++;
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Devuelve el valor de una determinada posicion
	 * 
	 * @param pos
	 *            posicionimport listaDoblementeEnlazadas.modelo.Nodo;
	 * @return el valor de tipo T
	 * @throws PosicionIlegalException
	 */
	public T getValor(int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos < tamanio) {
			T valor;
			if (pos == 0) {
				valor = cabeza.getValor();
				return valor;
			} else {
				Nodo<T> aux = cabeza;
				for (int i = 0; i <= pos - 1; i++) {
					aux = aux.getSiguiente();
				}
				valor = aux.getValor();
			}
			return valor;
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Elimina un nodo en una determinada posicion
	 * 
	 * @param pos
	 *            posicion
	 * @throws PosicionIlegalException
	 */
	public void remover(int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos < tamanio) {
			// El nodo a eliminar esta en la primera posicion
			if (pos == 0) {
				// Buscar el ultimo nodo para enlazarlo con la nueva cabeza
				Nodo<T> ultimo = cabeza;
				while (ultimo.getSiguiente() != cabeza) {
					ultimo = ultimo.getSiguiente();
				}
				// Desplaza la cabeza al siguiente nodo
				cabeza = cabeza.getSiguiente();
				// Enlaza el ultimo con la nueva cabeza
				ultimo.setSiguiente(cabeza);
			} else {
				// aux apunta justo al nodo anterior al que se elimina
				Nodo<T> aux = cabeza;
				for (int i = 0; i <= pos - 2; i++) {
					aux = aux.getSiguiente();
				}
				// prox apunta justo al nodo que se elimina
				Nodo<T> prox = aux.getSiguiente();
				aux.setSiguiente(prox.getSiguiente());
			}
			tamanio--;
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Elimina todos los nodos de la lista
	 */
	public void limpiar() {
		cabeza = null;
		tamanio = 0;
	}

	/**
	 * Imprime todos los nodos, sirve para probar que funciona bien
	 */

	public void imprimirTodo() {
		Nodo<T> aux = cabeza;
		System.out.println("Listado de todos los nodos:");
		Object nodo = new Object();
		while (aux.getSiguiente() != cabeza) {
			nodo = aux.getValor();
			System.out.println("" + nodo.toString());
			aux = aux.getSiguiente();
		}
	}
}
